package com.simplepart.samples.textformatting;

import org.junit.Test;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;
import com.simplepart.offbeat.Person;
import com.simplepart.offbeat.Gender;
import com.simplepart.offbeat.HitResolution;
import static junit.framework.Assert.assertEquals;

public abstract class DamageFormatterTest {

    protected abstract String getFormatterName();

     @Test
    public void shouldReturnFormattedMessage() {

        final Person attacker = new Person("Grognak the barbarian", Gender.Male);

        final Person[] targets = new Person[] {
            new Person("Big Bad Alien Thing", Gender.Unknown),
            new Person("Bad guy", Gender.Male),
            new Person("Bad gal", Gender.Female),
        };

        final HitResolution[] resolutions = new HitResolution[] {
            new HitResolution(attacker, targets[0], 1),
            new HitResolution(attacker, targets[1], 200),
            new HitResolution(attacker, targets[2], 200)
        };

        final String[] expectedMessages = new String[] {
            "Grognak the barbarian hits Big Bad Alien Thing in the groin for 1 point of damage, effectively ending its child-bearing days.",
            "Grognak the barbarian hits Bad guy in the groin for 200 points of damage, effectively ending his child-bearing days.",
            "Grognak the barbarian hits Bad gal in the groin for 200 points of damage, effectively ending her child-bearing days."
        };

        final DamageFormatter formatter = (DamageFormatter)getContext().getBean(getFormatterName());

        for (int index = 0; index < resolutions.length; index ++) {
            String actual = formatter.format(resolutions[index]);
            assertEquals(expectedMessages[index], actual);
        }
    }

    protected ApplicationContext getContext() {
        FileSystemXmlApplicationContext context = new FileSystemXmlApplicationContext("classpath:applicationContext.xml");
        context.refresh();

        return context;
    }
}
